-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local LOCATOR_LAYOUT_NAME = "locator";

local hidePosition =
{
  [  -1] = true, -- World/Cosmic
  [  13] = true, -- Kalimdor
  [  14] = true, -- Eastern Kingdoms
  [ 466] = true, -- Outland
  [ 485] = true, -- Northrend
  [ 613] = true, -- Vashj'ir
  [ 689] = true, -- Stranglethorn Vale
  [ 751] = true, -- Maelstrom
  [ 862] = true, -- Pandaria
  [ 962] = true, -- Draenor
  [1007] = true, -- Broken Isles
};

local locator = nil;
local mapLocation = nil;

-------------------------------------------------------------------------------
--             L  O  C  A  L     D  E  F  I  N  I  T  I  O  N  S
-------------------------------------------------------------------------------

local locatorUpdate;
 
-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to process events for the Locator frame
--
local function locatorOnClick(self, button)
  -- Local Variables
  local activeWindow = ChatEdit_GetActiveWindow();
  local currentText;
  local cursorPos;
  local numLetters;
  local posX;
  local posY;
  local zoneText = nil;

  -- Check there is an active window
  if (activeWindow ~= nil) then
    -- Get info from the active window
    currentText = activeWindow:GetText();
    cursorPos   = activeWindow:GetCursorPosition();
    numLetters  = activeWindow:GetNumLetters();

    -- Get the name of the (sub-)zone
    if (IsShiftKeyDown()) then
      zoneText = GetSubZoneText();
    end
    if ((zoneText == nil) or (zoneText == "")) then
      zoneText = GetRealZoneText();
    end

    -- Add the (sub-)zone and location to the active chat window
    if (zoneText ~= nil) then
      posX, posY = GetPlayerMapPosition("player");
      activeWindow:Insert(format("%s%s (%2.1f, %2.1f)%s",
        (((cursorPos > 0) and
          (currentText:sub(cursorPos, cursorPos) ~= " ")) and " " or ""),
        zoneText,
        (math.floor(posX*1000)/10), (math.floor(posY*1000)/10),
        (((cursorPos < numLetters) and
          (currentText:sub(cursorPos+1, cursorPos+1) ~= " ")) and " " or "")));
    end
  end

  return;
end

--
-- Function to display the tooltip for the Locator frame
--
local function locatorOnEnter(self)
  -- Show a tooltip for the Locator frame
  GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
  GameTooltip:SetText(L["Locator"], HIGHLIGHT_FONT_COLOR.r,
    HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, nil, true);
  GameTooltip:AddLine(L["LocatorTooltip"]);
  GameTooltip:SetMinimumWidth(135, true);
  GameTooltip:Show();

  return;
end

--
-- Function to process events for the Locator frame
--
local function locatorOnEvent(self, event, ...)
  -- Process the event
  if (event == "PET_BATTLE_OPENING_START") then
    self:Hide();
  elseif (event == "PET_BATTLE_CLOSE") then
    self:Show();
    NigglesLocatorShow(NigglesDBC.settings.LocatorShown);
  end

  return;
end

--
-- Function called when the user stops dragging the Locator frame
--
local function locatorOnDragStop(self)
  -- Stop moving the Locator frame and make sure its position is valid
  self:StopMovingOrSizing();
  ValidateFramePosition(self);

  -- Save the Locator frame's position
  L.layoutSave(LOCATOR_LAYOUT_NAME, self, false);

  return;
end

--
-- Function to hide the tooltip for the Locator frame
--
local function locatorOnLeave(self)
  -- Hide the tooltip
  GameTooltip:SetMinimumWidth(0, false);
  GameTooltip:Hide();

  return;
end

--
-- Function called when the Locator frame is shown
--
local function locatorOnShow(self)
  -- Restore the locator's position
  L.layoutRestore(LOCATOR_LAYOUT_NAME, self, false);

  -- Start updating the locator
  locatorUpdate();

  return
end

--
-- Function to update the coordinates displayed in the Locator frame
--
locatorUpdate = function()
  -- Check if the Locator is visible
  if (locator:IsShown()) then
    -- Local Variables
    local posX, posY = GetPlayerMapPosition("player");
    local currentMapId = GetCurrentMapAreaID();

    -- Check if the player's position can be displayed, based on current map
    if ((currentMapId == Niggles.zoneId) and
        (posX ~= nil) and (posX ~= 0) and
        (posY ~= nil) and (posY ~= 0)) then
      -- Adjust the player's current position
      posX = math.floor(posX*1000)/10;
      posY = math.floor(posY*1000)/10;

      -- Update the Locator's X position, if required
      if (posX ~= locator.lastX) then
        locator.posX:SetFormattedText("%2.1f,", posX);
        locator.lastX = posX;
      end

      -- Update the Locator's Y position, if required
      if (posY ~= locator.lastY) then
        locator.posY:SetFormattedText("%2.1f", posY);
        locator.lastY = posY;
      end
    elseif ((currentMapId == Niggles.zoneId) and
            ((posX == 0) and (posY == 0)) and
            (IsInInstance()) and
            (not WorldMapFrame:IsShown())) then
      -- Set the map to the current level of the instance's map
      SetMapToCurrentZone();
    elseif ((locator.lastX ~= -1) or (locator.lastY ~= -1)) then
      -- Display no position because the position is inaccurate
      locator.posX:SetText("");
      locator.posY:SetText("");
      locator.lastX = -1;
      locator.lastY = -1;
    end

    -- Update the locator again after a short while
    C_Timer.After(0.1, locatorUpdate);
  end

  return;
end

--
-- Function to update the coordinates displayed in the Map Locator frame
--
local function mapLocationOnUpdate(self, elapsed)
  -- Local Variables
   local cursorX = -1;
   local cursorY = -1;

  -- Check the locator is shown
  if (NigglesDBC.settings.UiMapLocation == true) then
    -- Check the mouse is over the map
    if ((WorldMapButton:IsMouseOver()) and
        (hidePosition[GetCurrentMapAreaID()] ~= true)) then
      -- Get the position of the mouse
      local mapCentreX, mapCentreY = WorldMapButton:GetCenter();
      local mapWidth = WorldMapButton:GetWidth();
      local mapHeight = WorldMapButton:GetHeight();
      local mapScale = WorldMapButton:GetEffectiveScale();

      -- Get and convert the cursor's position into a map position
      cursorX, cursorY = GetCursorPosition();
      cursorX = ((cursorX/mapScale)-(mapCentreX-(mapWidth/2)))/mapWidth;
      cursorY = (mapCentreY+(mapHeight/2)-(cursorY/mapScale))/mapHeight;
    end

    -- Check if the cursor position should be displayed
    if ((cursorX >= 0) and (cursorY >= 0)) then
      -- Change the cursor position to be 1 decimal place
      cursorX = math.floor(cursorX*1000)/10;
      cursorY = math.floor(cursorY*1000)/10;

      -- Update the Map Locator's X position, if required
      if (cursorX ~= self.lastX) then
        self.lastX = cursorX;
        self.posX:SetFormattedText("%2.01f,", cursorX);
      end

      -- Update the Map Locator's Y position, if required
      if (cursorY ~= self.lastY) then
        self.lastY = cursorY;
        self.posY:SetFormattedText("%2.01f", cursorY);
      end
    elseif ((self.lastX ~= -1) or (self.lastY ~= -1)) then
      -- Clear the Map Locator's position
      self.posX:SetText("");
      self.posY:SetText("");
      self.lastX = -1;
      self.lastY = -1;
    end
  end

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to enable/disable the Map Location frame
--
function L.mapLocationEnable()
  -- Check if the Map Location frame needs to be created
  if ((mapLocation == nil) and
      (NigglesDBC.settings.UiMapLocation == true)) then
    -- Create the Map Location frame
    mapLocation = CreateFrame("FRAME", nil, WorldMapFrame,
       "NigglesMapLocationTemplate");
    mapLocation:SetFrameLevel(WorldMapFrame.BorderFrame:GetFrameLevel()+1);
    mapLocation.lastX = 0;
    mapLocation.lastY = 0;

    -- Set the frame's script handlers
    mapLocation:SetScript("OnUpdate", mapLocationOnUpdate);
  end

  -- Show/Hide the Map Location frame, if required
  if (mapLocation ~= nil) then
    mapLocation:SetShown(NigglesDBC.settings.UiMapLocation);
  end

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to toggle the displaying of the Locator frames
--
function NigglesLocatorShow(show)
  -- Local Variables
  local frame;

  -- Work out the correct state for the Locator frames
  if (show ~= nil) then
    NigglesDBC.settings.LocatorShown = show;
  else
    NigglesDBC.settings.LocatorShown = not NigglesDBC.settings.LocatorShown;
  end

  -- Check if the locator frame needs to be created
  if ((NigglesDBC.settings.LocatorShown) and (locator == nil)) then
    -- Create the locator frame
    locator = CreateFrame("BUTTON", nil, UIParent,
       "NigglesLocatorTemplate");

    -- Register the frame for drag operations
    locator:RegisterForDrag("LeftButton");

    -- Set the frame's script handlers
    locator:SetScript("OnClick",    locatorOnClick);
    locator:SetScript("OnDragStop", locatorOnDragStop);
    locator:SetScript("OnEnter",    locatorOnEnter);
    locator:SetScript("OnEvent",    locatorOnEvent);
    locator:SetScript("OnLeave",    locatorOnLeave);
    locator:SetScript("OnShow",     locatorOnShow);

    -- Register for some events
    locator:RegisterEvent("PET_BATTLE_OPENING_START");
    locator:RegisterEvent("PET_BATTLE_CLOSE");

    -- Initialise the last position displayed
    locator.lastX = 0;
    locator.lastY = 0;
  end

  -- Show/Hide the Locator frame, if required
  if ((locator ~= nil) and (not C_PetBattles.IsInBattle())) then
    locator:SetShown(NigglesDBC.settings.LocatorShown);
  end

  return;
end
